@@ -1,47 +1,44 @@
-/*#################################################################################
-##
-##   R package Copula by Jun Yan and Ivan Kojadinovic Copyright (C) 2008, 2009
-##
-##   This file is part of the R package copula.
-##
-##   The R package copula is free software: you can redistribute it and/or modify
-##   it under the terms of the GNU General Public License as published by
-##   the Free Software Foundation, either version 3 of the License, or
-##   (at your option) any later version.
-##
-##   The R package copula is distributed in the hope that it will be useful,
-##   but WITHOUT ANY WARRANTY; without even the implied warranty of
-##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-##   GNU General Public License for more details.
-##
-##   You should have received a copy of the GNU General Public License
-##   along with the R package copula. If not, see <http://www.gnu.org/licenses/>.
-##
-#################################################################################*/
+/*
+  Copyright (C) 2012 Marius Hofert, Ivan Kojadinovic, Martin Maechler, and Jun Yan
+
+  This program is free software; you can redistribute it and/or modify it under
+  the terms of the GNU General Public License as published by the Free Software
+  Foundation; either version 3 of the License, or (at your option) any later
+  version.
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
+  details.
+
+  You should have received a copy of the GNU General Public License along with
+  this program; if not, see <http://www.gnu.org/licenses/>.
+*/
 
 
 /***********************************************************************
-  
- Testing wheter a copula belongs to the Extreme-Value class
- 
+
+ Testing whether a copula belongs to the Extreme-Value class
+
 ***********************************************************************/
 
 #include <R.h>
 #include <Rmath.h>
 #include <R_ext/Applic.h>
-#include "Anfun.h"
+
+#include "copula.h"
 
 /***********************************************************************
-  
- Empirical copula
- 
-***********************************************************************/
 
+ Empirical copula
+
+***********************************************************************/
+// FIXME: we have the empirical copula in about every *.c file ... -- use *one* !
 double ec(double *U, int n, int p, double *u, double o)
 {
   int i,j;
   double ind, res = 0.0;
-  
+
   for (i = 0; i < n; i++)
     {
       ind = 1.0;
@@ -53,49 +50,49 @@ double ec(double *U, int n, int p, double *u, double o)
 }
 
 /***********************************************************************
-  
+
  Derivative of the empirical copula
- 
+
 ***********************************************************************/
 
 double derec(double *U, int n, int p, double *u, double *v, double denom)
 {
-  return (ec(U, n, p, u, 0.0) - ec(U, n, p, v, 0.0)) / denom; 
+  return (ec(U, n, p, u, 0.0) - ec(U, n, p, v, 0.0)) / denom;
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test
- 
+
 ***********************************************************************/
 
-void evtest(double *U, int *n, int *p, double *g, int *m, 
-	    int *N, double *tg,  int *nt, double *s0, int *der2n, 
+void evtest(double *U, int *n, int *p, double *g, int *m,
+	    int *N, double *tg,  int *nt, double *s0, int *der2n,
 	    double *o, double *stat)
 {
   double *influ = Calloc((*n) * (*m) * (*nt), double);
   double *random = Calloc(*n, double);
- 
+
   double *u = Calloc(*p, double);
   double *v = Calloc(*p, double);
   double *ut = Calloc(*p, double);
   double *vt = Calloc(*p, double);
   double *der = Calloc(*p, double);
   double *dert = Calloc(*p, double);
- 
-  double t, ecterm, process, process2, mean, ind, indt, d, dt, denom,
+
+  double t, ecterm, process, mean, ind, indt, d, dt, denom,
     invsqrtn = 1.0/sqrt(*n), tmpu, tmpv, diff, ecut, ecu;
 
   int i, j, k, l, c;
 
-  for (c = 0; c < *nt; c++) 
+  for (c = 0; c < *nt; c++)
     {
       t = tg[c];
-      
+
       /* for each point of the grid */
-      for (j = 0; j < *m; j++) 
+      for (j = 0; j < *m; j++)
 	{
-      
+
 	  /* temporary arrays */
 	  for (k = 0; k < *p; k++)
 	    {
@@ -104,17 +101,17 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	      ut[k] =  R_pow(u[k], t);
 	      vt[k] =  ut[k];
 	    }
-	  
-	  ecterm = R_pow(ec(U, *n, *p, ut, 0.0), (1 - t)/t) / t; 
-	  
+
+	  ecterm = R_pow(ec(U, *n, *p, ut, 0.0), (1 - t)/t) / t;
+
 	  /* derivatives */
 	  for (k = 0; k < *p; k++)
 	    {
 	      /* bins = 2 invsqrtn */
-	      if (*der2n) 
+	      if (*der2n)
 		{
 		  denom = 2.0 * invsqrtn;
-		  
+
 		  /* u and v */
 		  if (u[k] < invsqrtn)
 		    {
@@ -130,7 +127,7 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 		      der[k] = derec(U, *n, *p, u, v, denom);
 		      u[k] = tmpu; v[k] = tmpv;
 		    }
-		  else 
+		  else
 		    {
 		      u[k] += invsqrtn; v[k] -= invsqrtn;
 		      der[k] = derec(U, *n, *p, u, v, denom);
@@ -152,30 +149,30 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 		      dert[k] = derec(U, *n, *p, ut, vt, denom);
 		      ut[k] = tmpu; vt[k] = tmpv;
 		    }
-		  else 
+		  else
 		    {
 		      ut[k] += invsqrtn; vt[k] -= invsqrtn;
 		      dert[k] = derec(U, *n, *p, ut, vt, denom);
 		      ut[k] -= invsqrtn; vt[k] += invsqrtn;
 		    }
-		} 
+		}
 	      else /* bins decreasing near 0 and 1 */
 		{
 		  u[k] += invsqrtn; v[k] -= invsqrtn;
-		  denom = MIN(u[k], 1.0) - MAX(v[k], 0.0); 
+		  denom = MIN(u[k], 1.0) - MAX(v[k], 0.0);
 		  der[k] = derec(U, *n, *p, u, v, denom);
 		  u[k] -= invsqrtn; v[k] += invsqrtn;
-		  
+
 		  ut[k] += invsqrtn; vt[k] -= invsqrtn;
 		  denom = MIN(ut[k], 1.0) - MAX(vt[k], 0.0);
 		  dert[k] = derec(U, *n, *p, ut, vt, denom);
 		  ut[k] -= invsqrtn; vt[k] += invsqrtn;
 		}
 	    }
-	  
+
 	  /* for each pseudo-obs */
-	  for (i = 0; i < *n; i++) 
-	    { 
+	  for (i = 0; i < *n; i++)
+	    {
 	      ind = 1.0;
 	      indt = 1.0;
 	      d = 0.0;
@@ -187,17 +184,17 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 		  d += der[k] * (U[i + k * (*n)] <= u[k]);
 		  dt += dert[k]  * (U[i + k * (*n)] <= ut[k]);
 		}
-	      influ[i + j * (*n) + c * (*n) * (*m)] 
+	      influ[i + j * (*n) + c * (*n) * (*m)]
 		= (ecterm * (indt - dt) - (ind - d)) * invsqrtn;
 	    }
 	}
     }
-  
+
   GetRNGstate();
-  
+
   /* generate N approximate realizations */
   for (l = 0; l < *N; l++)
-    { 
+    {
       /* generate n variates */
       mean = 0.0;
       for (i=0;i<*n;i++)
@@ -206,18 +203,18 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 	  mean += random[i];
 	}
       mean /= *n;
-      
+
       /* realization number l */
-      for (c = 0; c < *nt; c++) 
+      for (c = 0; c < *nt; c++)
 	{
 	  k = c + l * (*nt);
 	  s0[k] = 0.0;
 	  for (j = 0; j < *m; j++)
-	    { 
+	    {
 	      process = 0.0;
 	      for (i = 0; i < *n; i++)
 		process += (random[i] - mean) * influ[i + j * (*n) + c * (*n) * (*m)];
-	      
+
 	      s0[k] += process * process;
 	    }
 	  s0[k] /= *m;
@@ -227,12 +224,12 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
   PutRNGstate();
 
   /* test statistic */
-  for (c = 0; c < *nt; c++) 
+  for (c = 0; c < *nt; c++)
     {
       stat[c] = 0.0;
       t = tg[c];
       for (j = 0; j < *m; j++)
-	{ 
+	{
 	  for (k = 0; k < *p; k++)
 	    {
 	      u[k] = g[j + k * (*m)];
@@ -258,7 +255,7 @@ void evtest(double *U, int *n, int *p, double *g, int *m,
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An
  Derivatives based on Cn
 
@@ -268,7 +265,7 @@ double ecop(double *U, double *V, int n, double u, double v)
 {
   int i;
   double res = 0.0;
-  
+
   for (i = 0; i < n; i++)
       res += (U[i] <= u) * (V[i] <= v);
   return res/n;
@@ -277,7 +274,7 @@ double ecop(double *U, double *V, int n, double u, double v)
 double der1ec(double *U, double *V, int n, double u, double v)
 {
   double invsqrtn = 1.0 / sqrt(n);
-  return (ecop(U, V, n, u + invsqrtn, v) - ecop(U, V, n, u - invsqrtn, v)) 
+  return (ecop(U, V, n, u + invsqrtn, v) - ecop(U, V, n, u - invsqrtn, v))
     / (2.0 * invsqrtn);
 }
 
@@ -288,7 +285,7 @@ double der2ec(double *U, double *V, int n, double u, double v)
      / (2.0 * invsqrtn);
 }
 
-void evtestA(double *U, double *V, int *n, double *u, double *v, 
+void evtestA(double *U, double *V, int *n, double *u, double *v,
 	     int *m, int *CFG, int *N, double *s0)
 {
   double *influ = Calloc((*n) * (*m), double);
@@ -296,18 +293,18 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
   double *S = Calloc(*n, double);
   double *Sp = Calloc(*n, double);
   double *Sm = Calloc(*n, double);
-  double *T = Calloc(*n, double); 
-  double *Tp = Calloc(*n, double); 
-  double *Tm = Calloc(*n, double); 
- 
+  double *T = Calloc(*n, double);
+  double *Tp = Calloc(*n, double);
+  double *Tm = Calloc(*n, double);
+
   double *random = Calloc(*n, double);
 
-  double pu, pv, sum, d1, d2,  process, 
+  double pu, pv, sum, d1, d2,  process,
     mean, invsqrtn = 1.0 / sqrt(*n), minTkSi,  minSkTi,
     lb = 1.0 / (*n + 1.0), ub = *n / (*n + 1.0),
     A, cA0, cA1, t, loguv, loguvA, Aterm;
 
-  int i, j, k, l; 
+  int i, j, k, l;
 
   /* temporary arrays */
   for (i = 0; i < *n; i++)
@@ -328,12 +325,12 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
     }
   else
     {
-      cA0 = inv_A_Pickands(*n, S, T, 0.0); 
+      cA0 = inv_A_Pickands(*n, S, T, 0.0);
       cA1 = inv_A_Pickands(*n, S, T, 1.0);
     }
 
   /* for each point of the grid */
-  for (j = 0; j < *m; j++) 
+  for (j = 0; j < *m; j++)
     {
       loguv = log(u[j] * v[j]);
 
@@ -354,7 +351,7 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 	}
       else
 	{
-	  A = 1.0 / (inv_A_Pickands(*n, S, T, t) 
+	  A = 1.0 / (inv_A_Pickands(*n, S, T, t)
 		     - (1.0 - t) * (cA0  - 1.0)
 		     - t * (cA1 - 1.0));
 	  loguvA = loguv * A;
@@ -369,22 +366,22 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 	      minTkSi = MIN(pv * T[k], pu * S[i]);
 	      minSkTi = MIN(pu * S[k], pv * T[i]);
 	      if (*CFG)
-		sum += -log(MIN(pu * Sm[k], minTkSi)) 
+		sum += -log(MIN(pu * Sm[k], minTkSi))
 		  + log(MIN(pu * Sp[k], minTkSi))
-		  - log(MIN(pv * Tm[k], minSkTi)) 
+		  - log(MIN(pv * Tm[k], minSkTi))
 		  + log(MIN(pv * Tp[k], minSkTi));
 	      else
 		sum += MIN(pu * Sm[k], minTkSi) - MIN(pu * Sp[k], minTkSi)
 		  + MIN(pv * Tm[k], minSkTi) - MIN(pv * Tp[k], minSkTi);
 	    }
 	  sum *= invsqrtn / 2.0;
-	  
+
 	  if (*CFG)
-	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j]) 
-	      - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j]) 
+	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
+	      - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
 	      - Aterm * (-log(MIN(pu * S[i], pv * T[i])) - sum);
 	  else
-	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j]) 
+	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
 	      - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
 	      + Aterm * (MIN(pu * S[i], pv * T[i]) - sum);
 
@@ -405,18 +402,18 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 	  mean += random[i];
 	}
       mean /= *n;
-      
+
       /* realization number l */
       s0[l] = 0.0;
       for (j = 0; j < *m; j++)
-	{ 
+	{
 	  process = 0.0;
 	  for (i = 0; i < *n; i++)
 	    process += (random[i] - mean) * influ[i + j * (*n)];
 
 	  s0[l] += process * process;
 	}
-      s0[l] /= *m; 
+      s0[l] /= *m;
     }
 
   PutRNGstate();
@@ -424,7 +421,7 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 
   Free(influ);
   Free(random);
-  
+
   Free(S);
   Free(T);
   Free(Sp);
@@ -434,14 +431,14 @@ void evtestA(double *U, double *V, int *n, double *u, double *v,
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An
  Derivatives based on An
 
 ***********************************************************************/
 
-double intgr(double x, double termUt, double termVt, double powUt, double powVt, 
-	     double U, double V, double t, double n) 
+double intgr(double x, double termUt, double termVt, double powUt, double powVt,
+	     double U, double V, double t, double n)
 {
   double x1t = R_pow(x,1-t), xt = R_pow(x,t);
   double indUt = (U <= x1t) - (int)(x1t * (n+1)) / n;
@@ -450,30 +447,30 @@ double intgr(double x, double termUt, double termVt, double powUt, double powVt,
   if (indUt) res +=  termUt * R_pow(x,powUt) * indUt / xlogx;
   if (indVt) res += termVt * R_pow(x,powVt) * indVt / xlogx;
   return res;
-} 
+}
 
-void vec_intgr(double *x, int n, void *ex) 
+void vec_intgr(double *x, int n, void *ex)
 {
   int i;
   double *arg = ex;
-  for (i = 0; i < n; i++) 
-    x[i] = intgr(x[i], arg[0], arg[1], arg[2], arg[3], 
+  for (i = 0; i < n; i++)
+    x[i] = intgr(x[i], arg[0], arg[1], arg[2], arg[3],
 		 arg[4], arg[5], arg[6], arg[7]);
   return;
 }
 
-void evtestA_derA(double *U, double *V, int *n, double *u, double *v, 
+void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 	     int *m, int *CFG, int *N, double *s0)
 {
   double *influ = Calloc((*n) * (*m), double);
   double *influ2 = Calloc((*n) * (*m), double);
 
   double *S = Calloc(*n, double);
-  double *T = Calloc(*n, double); 
- 
+  double *T = Calloc(*n, double);
+
   double *random = Calloc(*n, double);
 
-  /* for numerical integration: begin */ 
+  /* for numerical integration: begin */
   double result, abserr;
   int last, neval, ier;
   int limit=100;
@@ -482,16 +479,16 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
   int lenw = 4 * limit;
   int *iwork = Calloc(limit, int);
   double *work = Calloc(lenw, double);
-  double ex[8], lower = 0.0, upper = 1.0; 
+  double ex[8], lower = 0.0, upper = 1.0;
   /* lower = 1.0 / (*n + 1), upper = *n / (*n + 1.0); */
-  /* for numerical integration: end */ 
+  /* for numerical integration: end */
 
-  double pu, pv, d1, d2,  process, 
-    mean, invsqrtn = 1.0 / sqrt(*n), 
-    A, cA0, cA1, t, tp, tm, uv, loguv, loguvA, 
+  double pu, pv, d1, d2,  process,
+    mean, invsqrtn = 1.0 / sqrt(*n),
+    A, cA0, cA1, t, tp, tm, uv, loguv, loguvA,
     Aterm, dAt, Ap, Am, termUt, termVt, powUt, powVt;
 
-  int i, j, l; 
+  int i, j, l;
 
   /* temporary arrays */
   for (i = 0; i < *n; i++)
@@ -508,12 +505,12 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
     }
   else
     {
-      cA0 = inv_A_Pickands(*n, S, T, 0.0); 
+      cA0 = inv_A_Pickands(*n, S, T, 0.0);
       cA1 = inv_A_Pickands(*n, S, T, 1.0);
     }
 
   /* for each point of the grid */
-  for (j = 0; j < *m; j++) 
+  for (j = 0; j < *m; j++)
     {
       uv = u[j] * v[j];
       loguv = log(uv);
@@ -545,16 +542,16 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 	}
       else
 	{
-	  A = 1.0 / (inv_A_Pickands(*n, S, T, t) 
+	  A = 1.0 / (inv_A_Pickands(*n, S, T, t)
 		     - (1.0 - t) * (cA0  - 1.0)
 		     - t * (cA1 - 1.0));
 	  loguvA = loguv * A;
 	  Aterm = exp(loguvA) * loguvA * A;
 
-	  Ap = 1.0 / (inv_A_Pickands(*n, S, T, tp) 
+	  Ap = 1.0 / (inv_A_Pickands(*n, S, T, tp)
 		       - (1.0 - tp) * (cA0  - 1.0)
 		       - tp * (cA1 - 1.0));
-	  Am = 1.0 / (inv_A_Pickands(*n, S, T, tm) 
+	  Am = 1.0 / (inv_A_Pickands(*n, S, T, tm)
 		       - (1.0 - tm) * (cA0  - 1.0)
 		       - tm * (cA1 - 1.0));
 	  dAt = (Ap - Am) / (2.0 * invsqrtn);
@@ -576,11 +573,11 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 	{
 	  if (*CFG)
 	    {
-	      ex[0] = termUt; ex[1] = termVt; 
+	      ex[0] = termUt; ex[1] = termVt;
 	      ex[2] = powUt; ex[3] = powVt;
 	      ex[4] = U[i]; ex[5] = V[i];
 	      ex[6] = t; ex[7] = *n;
-	      
+
 	      Rdqags(vec_intgr, (void *)ex, &lower, &upper,
 		     &abstol,  &reltol,
 		     &result,  &abserr,  &neval,  &ier,
@@ -589,19 +586,19 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 
 	      /* if (ier)
 		 Rprintf("%lf %lf %d\n", result, abserr, ier); */
-	      
-	      influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j]) 
-		- d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j]) 
+
+	      influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
+		- d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
 		+ Aterm * log(MIN(pu * S[i], pv * T[i]));
-	      influ2[i + j * (*n)] =  Aterm * result; 
+	      influ2[i + j * (*n)] =  Aterm * result;
 	    }
 	  else
-	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j]) 
+	    influ[i + j * (*n)] = (U[i] <= u[j]) * (V[i] <= v[j])
 	      - d1 * (U[i] <= u[j]) - d2 * (V[i] <= v[j])
-	      + Aterm * (MIN(pu * S[i], pv * T[i])  
+	      + Aterm * (MIN(pu * S[i], pv * T[i])
 			 - termUt * (1.0 - R_pow(U[i],powUt))
 			 - termVt * (1.0 - R_pow(V[i],powVt)));
-	  
+
 	  influ[i + j * (*n)] *= invsqrtn;
 	  influ2[i + j * (*n)] *=  invsqrtn;
 	}
@@ -620,22 +617,22 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
 	  mean += random[i];
 	}
       mean /= *n;
-      
+
       /* realization number l */
       s0[l] = 0.0;
       for (j = 0; j < *m; j++)
-	{ 
+	{
 	  process = 0.0;
 	  for (i = 0; i < *n; i++)
 	    if (*CFG)
-	      process += (random[i] - mean) * influ[i + j * (*n)] 
+	      process += (random[i] - mean) * influ[i + j * (*n)]
 		+ random[i] * influ2[i + j * (*n)];
 	    else
 	      process += (random[i] - mean) * influ[i + j * (*n)];
-	  
+
 	  s0[l] += process * process;
 	}
-      s0[l] /= *m; 
+      s0[l] /= *m;
     }
 
   PutRNGstate();
@@ -644,20 +641,19 @@ void evtestA_derA(double *U, double *V, int *n, double *u, double *v,
   Free(influ);
   Free(influ2);
   Free(random);
-  
+
   Free(S);
   Free(T);
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An
  Test statistic
 
 ***********************************************************************/
 
-    
-void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,  
+void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
 		  int *CFG, double *stat, double *offset)
 {
   int i, j;
@@ -670,7 +666,7 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
       S[i] = - log(U[i]);
       T[i] = - log(V[i]);
     }
-  
+
   /* correction terms */
   if (*CFG)
     {
@@ -682,7 +678,7 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
       cA0 = inv_A_Pickands(*n, S, T, 0.0);
       cA1 = inv_A_Pickands(*n, S, T, 1.0);
     }
-  
+
   for (j = 0; j < *m; j++)
     {
       loguv = log(u[j] *v[j]);
@@ -698,13 +694,13 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
       if (*offset < 0.0)
 	diff = ecop(U, V, *n, u[j], v[j]) - exp(loguv * Aj);
       else
-	diff = ecop(U, V, *n, u[j], v[j]) * (*n) / (*n+1) 
+	diff = ecop(U, V, *n, u[j], v[j]) * (*n) / (*n+1)
 	  + (*offset)/(*n+1) - exp(loguv * Aj);
-	
-      
+
+
       s += diff * diff;
     }
-  
+
   *stat = s * (*n) / *m;
 
   Free(S);
@@ -713,18 +709,18 @@ void evtestA_stat(double *U, double *V, int *n, double *u, double *v, int *m,
 
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An CFG - An Pickands
  Derivatives based on Cn
 
 ***********************************************************************/
 
-void evTestAA(double *U, double *V, int *n, double *t, int *m, 
-	      int *N, double *s0)
+void evTestAA_C(double *U, double *V, int *n, double *t, int *m,
+		int *N, double *s0)
 {
   double *influ = Calloc((*n) * (*m), double);
   double *random = Calloc(*n, double);
- 
+
   double *S = Calloc(*n, double);
   double *T = Calloc(*n, double);
   double *Sp = Calloc(*n, double);
@@ -732,12 +728,12 @@ void evTestAA(double *U, double *V, int *n, double *t, int *m,
   double *Sm = Calloc(*n, double);
   double *Tm = Calloc(*n, double);
 
-  double sumcfg, sump, Acfg, Ap, process, mean, 
-    invsqrtn = 1.0/sqrt(*n), minTkSi, minSkTi, 
+  double sumcfg, sump, Acfg, Ap, process, mean,
+    invsqrtn = 1.0/sqrt(*n), minTkSi, minSkTi,
     lb = 1.0 / (*n + 1.0), ub = *n / (*n + 1.0),
     cA0cfg, cA1cfg, cA0p, cA1p;
 
-  int i, j, k, l; 
+  int i, j, k, l;
 
   /* temporary arrays */
   for (i = 0; i < *n; i++)
@@ -749,37 +745,37 @@ void evTestAA(double *U, double *V, int *n, double *t, int *m,
       Sm[i] = - log(MAX(U[i] - invsqrtn, lb));
       Tm[i] = - log(MAX(V[i] - invsqrtn, lb));
     }
-  
+
   /* correction terms */
   cA0cfg = log_A_CFG(*n, S, T, 0.0);
   cA1cfg = log_A_CFG(*n, S, T, 1.0);
-  cA0p = inv_A_Pickands(*n, S, T, 0.0); 
+  cA0p = inv_A_Pickands(*n, S, T, 0.0);
   cA1p = inv_A_Pickands(*n, S, T, 1.0);
-  
+
   /* for each point of the grid */
-  for (j = 0; j < *m; j++) 
+  for (j = 0; j < *m; j++)
     {
-     
+
       Acfg = exp(log_A_CFG(*n, S, T, t[j])
 		 - (1.0 - t[j]) * cA0cfg - t[j] * cA1cfg);
-      Ap = 1.0 / (inv_A_Pickands(*n, S, T, t[j]) 
+      Ap = 1.0 / (inv_A_Pickands(*n, S, T, t[j])
 		  - (1.0 - t[j]) * (cA0p  - 1.0)
 		  - t[j] * (cA1p - 1.0));
-      
+
       for (i = 0; i < *n; i++) /* for each pseudo-obs */
-	{ 
+	{
 	  sumcfg = 0.0;
 	  sump = 0.0;
 	  for (k = 0; k < *n; k++)
-	    { 
+	    {
 	      minTkSi = MIN(T[k] / t[j], S[i] / (1.0 - t[j]));
 	      minSkTi = MIN(S[k] / (1.0 - t[j]), T[i] / t[j]);
-	      
+
 	      sumcfg += - log(MIN(Sm[k] / (1.0 - t[j]), minTkSi))
 		+ log(MIN(Sp[k] / (1.0 - t[j]), minTkSi))
 		- log(MIN(Tm[k] / t[j], minSkTi))
 		+ log(MIN(Tp[k] / t[j], minSkTi));
-	      
+
 	      sump += MIN(Sm[k] / (1.0 - t[j]), minTkSi)
 		- MIN(Sp[k] / (1.0 - t[j]), minTkSi)
 		+ MIN(Tm[k] / t[j], minSkTi)
@@ -789,10 +785,10 @@ void evTestAA(double *U, double *V, int *n, double *t, int *m,
 	  sump *= invsqrtn / 2.0;
 
 	  /* influence term */
-	  influ[i + j * (*n)] = 
+	  influ[i + j * (*n)] =
 	    Acfg * (-log(MIN(S[i] / (1.0 - t[j]), T[i] / t[j])) - sumcfg)
-	    + Ap * Ap * (MIN(S[i] / (1.0 - t[j]), T[i] / t[j]) - sump); 
-	  
+	    + Ap * Ap * (MIN(S[i] / (1.0 - t[j]), T[i] / t[j]) - sump);
+
 	  influ[i + j * (*n)] *=  invsqrtn;
 	}
     }
@@ -810,18 +806,18 @@ void evTestAA(double *U, double *V, int *n, double *t, int *m,
 	  mean += random[i];
 	}
       mean /= (double)(*n);
-      
+
       /* realization number l */
       s0[l] = 0.0;
       for (j = 0; j < *m; j++)
-	{ 
+	{
 	  process = 0.0;
 	  for (i = 0; i < *n; i++)
 	    process += (random[i] - mean) * influ[i + j * (*n)];
 
 	  s0[l] += process * process;
 	}
-      s0[l] /= (double)(*m); 
+      s0[l] /= (double)(*m);
     }
 
   PutRNGstate();
@@ -838,15 +834,15 @@ void evTestAA(double *U, double *V, int *n, double *t, int *m,
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An CFG - An Pickands
  Derivatives based on An
 
 ***********************************************************************/
 
-double ntgrand(double x, double termUcfg, double termVcfg, 
-	      double powUcfg, double powVcfg, double U, double V, 
-	      double t, double n) 
+double ntgrand(double x, double termUcfg, double termVcfg,
+	      double powUcfg, double powVcfg, double U, double V,
+	      double t, double n)
 {
   double x1t = R_pow(x,1-t), xt = R_pow(x,t);
   double indUcfg = (U <= x1t) - (int)(x1t * (n+1)) / n;
@@ -855,36 +851,36 @@ double ntgrand(double x, double termUcfg, double termVcfg,
   if (indUcfg) res += termUcfg * R_pow(x,powUcfg) * indUcfg / xlogx;
   if (indVcfg) res += termVcfg * R_pow(x,powVcfg) * indVcfg / xlogx;
   return res;
-} 
+}
 
-void vec_ntgrand(double *x, int n, void *ex) 
+void vec_ntgrand(double *x, int n, void *ex)
 {
   int i;
   double *arg = ex;
-  for (i = 0; i < n; i++) 
-    x[i] = ntgrand(x[i], arg[0], arg[1], arg[2], arg[3], 
+  for (i = 0; i < n; i++)
+    x[i] = ntgrand(x[i], arg[0], arg[1], arg[2], arg[3],
 		    arg[4], arg[5], arg[6], arg[7]);
   return;
 }
 
-void evTestAA_derA(double *U, double *V, int *n, double *t, int *m, 
+void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
 		   int *N, double *s0)
 {
   double *influ = Calloc((*n) * (*m), double);
   double *influ2 = Calloc((*n) * (*m), double);
   double *random = Calloc(*n, double);
- 
+
   double *S = Calloc(*n, double);
   double *T = Calloc(*n, double);
 
-  double Acfg, Ap, Acfgm, Acfgp, Apm, App, dAcfg, dAp, 
+  double Acfg, Ap, Acfgm, Acfgp, Apm, App, dAcfg, dAp,
     tj, tjp, tjm, termUcfg, termVcfg, termUp, termVp,
-    powUcfg, powVcfg, powUp, powVp, process, mean, 
+    powUcfg, powVcfg, powUp, powVp, process, mean,
     invsqrtn = 1.0/sqrt(*n), cA0cfg, cA1cfg, cA0p, cA1p;
 
-  int i, j, l; 
+  int i, j, l;
 
-  /* for numerical integration: begin */ 
+  /* for numerical integration: begin */
   double result, abserr;
   int last, neval, ier;
   int limit=100;
@@ -893,9 +889,9 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
   int lenw = 4 * limit;
   int *iwork = Calloc(limit, int);
   double *work = Calloc(lenw, double);
-  double ex[8], lower = 0.0, upper = 1.0; 
+  double ex[8], lower = 0.0, upper = 1.0;
   /* lower = 1.0 / (*n + 1), upper = *n / (*n + 1.0); */
-  /* for numerical integration: end */ 
+  /* for numerical integration: end */
 
   /* temporary arrays */
   for (i = 0; i < *n; i++)
@@ -903,15 +899,15 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
       S[i] = - log(U[i]);
       T[i] = - log(V[i]);
     }
-  
+
   /* correction terms */
   cA0cfg = log_A_CFG(*n, S, T, 0.0);
   cA1cfg = log_A_CFG(*n, S, T, 1.0);
-  cA0p = inv_A_Pickands(*n, S, T, 0.0); 
+  cA0p = inv_A_Pickands(*n, S, T, 0.0);
   cA1p = inv_A_Pickands(*n, S, T, 1.0);
-  
+
   /* for each point of the grid */
-  for (j = 0; j < *m; j++) 
+  for (j = 0; j < *m; j++)
     {
       tj = t[j];
       tjp = tj + invsqrtn;
@@ -928,15 +924,15 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
       termVcfg = Acfg + (1.0 - tj) * dAcfg;
       powUcfg = Acfg + tj - 1.0;
       powVcfg = Acfg - tj;
-      
-      
-      Ap = 1.0 / (inv_A_Pickands(*n, S, T, tj) 
+
+
+      Ap = 1.0 / (inv_A_Pickands(*n, S, T, tj)
 		  - (1.0 - tj) * (cA0p  - 1.0)
 		  - t[j] * (cA1p - 1.0));
-      App = 1.0 / (inv_A_Pickands(*n, S, T, tjp) 
+      App = 1.0 / (inv_A_Pickands(*n, S, T, tjp)
 		   - (1.0 - tjp) * (cA0p  - 1.0)
 		   - tjp * (cA1p - 1.0));
-      Apm = 1.0 / (inv_A_Pickands(*n, S, T, tjm) 
+      Apm = 1.0 / (inv_A_Pickands(*n, S, T, tjm)
 		   - (1.0 - tjm) * (cA0p  - 1.0)
 		   - tjm * (cA1p - 1.0));
       dAp = (App - Apm) / (2.0 * invsqrtn);
@@ -944,38 +940,38 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
       termVp = (Ap + (1.0 - tj) * dAp) / (Ap - tj);
       powUp = (Ap + tj - 1.0) / (1.0 - tj);
       powVp = (Ap - tj) / tj;
-      
+
       for (i = 0; i < *n; i++) /* for each pseudo-obs */
-	{ 
-	  ex[0] = termUcfg; ex[1] = termVcfg; 
+	{
+	  ex[0] = termUcfg; ex[1] = termVcfg;
 	  ex[2] = powUcfg; ex[3] = powVcfg;
 	  ex[4] = U[i]; ex[5] = V[i];
 	  ex[6] = t[j]; ex[7] = *n;
-	  
+
 	  Rdqags(vec_ntgrand, (void *)ex, &lower, &upper,
 		 &abstol,  &reltol,
 		 &result,  &abserr,  &neval,  &ier,
 		 &limit,  &lenw, &last,
 		 iwork, work);
-	  
+
 	  /*if (ier)
 	    Rprintf("%lf %lf %d\n", result, abserr, ier); */
-	  
-	  /* influence matrices */	      
+
+	  /* influence matrices */
 	  influ[i + j * (*n)] = - Acfg * log(MIN(S[i] / (1.0 - t[j]), T[i] / t[j]))
 	    + Ap * Ap * (MIN(S[i]/ (1.0 - t[j]), T[i] / t[j])
 			 - termUp * (1.0 - R_pow(U[i],powUp))
-			 - termVp * (1.0 - R_pow(V[i],powVp))); 
-	  
+			 - termVp * (1.0 - R_pow(V[i],powVp)));
+
 	  influ2[i + j * (*n)] =  - Acfg * result;
-	  
+
 	  influ[i + j * (*n)] *=  invsqrtn;
 	  influ2[i + j * (*n)] *=  invsqrtn;
 	}
     }
-      
+
   GetRNGstate();
-  
+
   /* generate N approximate realizations */
   for (l = 0; l < *N; l++)
     {
@@ -987,19 +983,19 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
 	  mean += random[i];
 	}
       mean /= (double)(*n);
-      
+
       /* realization number l */
       s0[l] = 0.0;
       for (j = 0; j < *m; j++)
-	{ 
+	{
 	  process = 0.0;
 	  for (i = 0; i < *n; i++)
-	    process += (random[i] - mean) * influ[i + j * (*n)] 
+	    process += (random[i] - mean) * influ[i + j * (*n)]
 	      + random[i] * influ2[i + j * (*n)];
 
 	  s0[l] += process * process;
 	}
-      s0[l] /= (double)(*m); 
+      s0[l] /= (double)(*m);
     }
 
   PutRNGstate();
@@ -1015,13 +1011,13 @@ void evTestAA_derA(double *U, double *V, int *n, double *t, int *m,
 }
 
 /***********************************************************************
-  
+
  Extreme-Value Test based on An CFG - An Pickands
  Test statistic
 
 ***********************************************************************/
 
-void evTestAA_stat(double *S, double *T, int *n, double *t, int *m, 
+void evTestAA_stat(double *S, double *T, int *n, double *t, int *m,
 		   double *stat)
 {
   int j;
@@ -1029,19 +1025,19 @@ void evTestAA_stat(double *S, double *T, int *n, double *t, int *m,
 
   /* correction terms */
   cA0cfg = log_A_CFG(*n, S, T, 0.0);
-  cA1cfg = log_A_CFG(*n, S, T, 1.0); 
-  cA0p = inv_A_Pickands(*n, S, T, 0.0); 
+  cA1cfg = log_A_CFG(*n, S, T, 1.0);
+  cA0p = inv_A_Pickands(*n, S, T, 0.0);
   cA1p = inv_A_Pickands(*n, S, T, 1.0);
-  
+
   for (j = 0; j < *m; j++)
-    { 
+    {
       diff = exp(log_A_CFG(*n, S, T, t[j])
 		 - (1.0 - t[j]) * cA0cfg - t[j] * cA1cfg)
 	- 1.0/(inv_A_Pickands(*n, S, T, t[j])
 	       - (1.0 - t[j]) * (cA0p  - 1.0)
 	       - t[j] * (cA1p - 1.0));
-      s += diff * diff;  
+      s += diff * diff;
     }
-  
+
   *stat = s * (double)(*n) / (double)(*m);
 }
